@@ -1,10 +1,10 @@
 from __future__ import absolute_import
-# Copyright (c) 2010-2016 openpyxl
+# Copyright (c) 2010-2017 openpyxl
 
 import re
 import warnings
 
-from openpyxl.compat import unicode
+from openpyxl.compat import unicode, safe_repr
 
 from openpyxl.worksheet.header_footer import HeaderFooter
 
@@ -50,8 +50,9 @@ class _WorkbookChild(object):
         self.title = title or self._default_title
         self.HeaderFooter = HeaderFooter()
 
+
     def __repr__(self):
-        return u'<{0} "{1}">'.format(self.__class__.__name__, self.title)
+        return '<{0} "{1}">'.format(self.__class__.__name__, safe_repr(self.title))
 
 
     @property
