@@ -1,5 +1,5 @@
 from __future__ import absolute_import
-# Copyright (c) 2010-2016 openpyxl
+# Copyright (c) 2010-2017 openpyxl
 
 """Workbook is the top-level container for all document information."""
 
@@ -126,8 +126,11 @@ class Workbook(object):
 
     @property
     def active(self):
-        """Get the currently active sheet"""
-        return self._sheets[self._active_sheet_index]
+        """Get the currently active sheet or None"""
+        try:
+            return self._sheets[self._active_sheet_index]
+        except IndexError:
+            pass
 
     @active.setter
     def active(self, value):
@@ -272,7 +275,7 @@ class Workbook(object):
         """
         Add a named style
         """
-        style.xfId = self._named_styles.add(style)
+        self._named_styles.append(style)
         style.bind(self)
 
 
@@ -363,3 +366,11 @@ class Workbook(object):
         cp = WorksheetCopy(source_worksheet=from_worksheet, target_worksheet=to_worksheet)
         cp.copy_worksheet()
         return to_worksheet
+
+
+    def close(self):
+        """
+        Close workbook file if open. Only affects read-only and write-only modes.
+        """
+        if hasattr(self, '_archive'):
+            self._archive.close()
